unit MainForm;

interface

uses
  OpenGL, Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, ComCtrls;

const
  { ClientWidth x ClientHeight x RGB. }
  Pixels = 534 * 637 * 3;

type
  TfrmMain = class(TForm)
    bStartOpenGL: TButton;
    bStopOpenGL: TButton;
    bEnd: TButton;
    bSaveAndConvert: TButton;
    bDrawGDI: TButton;
    Timer3: TTimer;
    procedure FormCreate (Sender : TObject);
    procedure bStartOpenGLClick(Sender: TObject);
    procedure bStopOpenGLClick(Sender: TObject);
    procedure bDrawGDIClick(Sender: TObject);
    procedure bEndClick(Sender: TObject);
    procedure bSaveAndConvertClick(Sender: TObject);
    procedure Timer3Timer(Sender: TObject);
  private
    procedure SetOrthographicProjection;
    procedure SetPerspectiveProjection;
    procedure DrawBackground;
    procedure DrawForeground;
  public
  end;

  PRGBArray = ^TRGBArray;
  TRGBArray = array [0..533] of TRGBTriple;

var
  frmMain : TfrmMain;
  Base, Rotation, Changer : GLUInt;
  ToPrint : String;
  GMF : array [0..255] of GLYPHMETRICSFLOAT;
  DC : HDC;
  RC : HGLRC;
  ScreenData : array [0..Pixels] of Byte;
  ScreenBitmap : TBitmap;

  procedure glBindTexture (Target : GLEnum; Texture : GLUInt); stdcall; external OpenGL32;


implementation


{$R *.DFM}


procedure BuildFont;
var
  Font : HFont;
begin
  Base := glGenLists (256);
  Font := CreateFont (-12,-12,0,0,0,0,0,0,ANSI_CHARSET,OUT_TT_PRECIS,CLIP_DEFAULT_PRECIS,ANTIALIASED_QUALITY,FF_DONTCARE or DEFAULT_PITCH,'Arial');
  SelectObject (DC,Font);
  wglUseFontOutlines (DC,0,255,Base,0,0.2,WGL_FONT_POLYGONS,@GMF);
end;


procedure glPrint ( Text : String
                  );
begin
  if (Text <> '') then
  begin
    glTranslatef (-0.22,0,0);
    glPushAttrib (GL_LIST_BIT);
      glListBase (base);
      glCallLists (Length (Text),GL_UNSIGNED_BYTE,PChar (Text));
    glPopAttrib;
  end;
end;


procedure setupPixelFormat ( DC : HDC
                           );
const
   pfd : TPIXELFORMATDESCRIPTOR = (
        nSize : SizeOf (TPIXELFORMATDESCRIPTOR);
        nVersion : 1;
        dwFlags : PFD_SUPPORT_OPENGL or PFD_DRAW_TO_WINDOW or PFD_DOUBLEBUFFER;
        iPixelType : PFD_TYPE_RGBA;
        cColorBits : 24;
        cRedBits : 0;
        cRedShift : 0;
        cGreenBits : 0;
        cGreenShift : 0;
        cBlueBits : 0;
        cBlueShift : 0;
        cAlphaBits : 0;
        cAlphaShift : 0;
        cAccumBits : 0;
        cAccumRedBits : 0;
        cAccumGreenBits : 0;
        cAccumBlueBits : 0;
        cAccumAlphaBits : 0;
        cDepthBits : 16;
        cStencilBits : 0;
        cAuxBuffers : 0;
        iLayerType : PFD_MAIN_PLANE;
        bReserved : 0;
        dwLayerMask : 0;
        dwVisibleMask : 0;
        dwDamageMask : 0;);
var
  PixelFormat : Integer;
begin
   PixelFormat := ChoosePixelFormat (DC,@pfd);
   if (pixelFormat = 0) then
   begin
     Exit;
   end;

   if (SetPixelFormat (DC,PixelFormat,@pfd) <> True) then
   begin
     Exit;
   end;
end;


procedure TfrmMain.FormCreate ( Sender : TObject
                              );
begin
  DC := GetDC (frmMain.Handle);
  SetupPixelFormat (DC);
  RC := wglCreateContext (DC);
  wglMakeCurrent (DC,RC);

  glShadeModel (GL_SMOOTH);
  glClearColor (0,0,0,0);
  glEnable (GL_DEPTH_TEST);
  glHint (GL_PERSPECTIVE_CORRECTION_HINT,GL_NICEST);
  glEnable (GL_LIGHT0);
  glEnable (GL_LIGHTING);
  glEnable (GL_COLOR_MATERIAL);
  glPixelStorei (GL_UNPACK_ALIGNMENT,1);

  BuildFont;

  Rotation := 30;
  Changer := 0;
  ToPrint := '3';
  glColor3f (1,0,0);
end;


procedure TfrmMain.bStartOpenGLClick ( Sender : TObject
                                     );
begin
  { Commence OpenGL rendering... }
  Timer3.Enabled := True;
end;


procedure TfrmMain.bStopOpenGLClick ( Sender : TObject
                                    );
begin
  Timer3.Enabled := False;
  { Replace what OpenGL has rendered with the saved TBitmap. }
  frmMain.Canvas.Draw (0,0,ScreenBitmap);
  ScreenBitmap.Free;
end;


procedure TfrmMain.bDrawGDIClick ( Sender : TObject
                                 );
begin
  with frmMain.Canvas do
  begin
    SetBkMode (Handle,Transparent);
    Pen.Color := clLime;
    MoveTo (0,0);
    LineTo (ClientWidth,0);
    Pen.Color := clRed;
    Pen.Width := 2;
    Ellipse (10,10,100,100);
    Pen.Color := clYellow;
    Rectangle (100,100,200,300);
    Pen.Color := clLime;
    Pen.Width := 3;
    MoveTo (75,75);
    LineTo (400,125);
    Pen.Color := clOlive;
    MoveTo (300,50);
    LineTo (50,400);
    Pen.Color := clBlue;
    Pen.Width := 3;
    Rectangle (250,300,500,500);
    Font.Name := 'Arial';
    Font.Size := 20;
    Font.Color := clWhite;
    Font.Style := Font.Style + [fsBold];
    TextOut (50,100,'Here is some text...');
    Font.Color := clLime;
    TextOut (20,400,'Here is some more text...');
    Brush.Color := clYellow;
    FillRect (Bounds (0,0,10,10));
    FillRect (Bounds (0,ClientHeight - 10,10,10));
    FillRect (Bounds (ClientWidth - 10,ClientHeight - 10,10,10));
    FillRect (Bounds (ClientWidth - 10,0,10,10));
    Brush.Color := clNone;
  end;
end;


procedure TfrmMain.bSaveAndConvertClick ( Sender : TObject
                                        );
var
  RowLoop, ColLoop, Index : Integer;
  ScanLine : PRGBArray;
begin
  { Create Bitmap and copy the TForm's contents to it. }
  ScreenBitmap := TBitmap.Create;
  ScreenBitmap.PixelFormat := pf24bit;
  ScreenBitmap.Width := frmMain.ClientWidth;
  ScreenBitmap.Height := frmMain.ClientHeight;
  ScreenBitmap.Canvas.CopyRect (ClientRect,Canvas,ClientRect);

  { Copy the TBitmap's pixels to an array of Byte that glReadPixels can use. }
  Index := 0;
  for RowLoop := ScreenBitmap.Height - 1 downto 0 do
  begin
    ScanLine := ScreenBitmap.ScanLine[RowLoop];
    for ColLoop := 0 to ScreenBitmap.Width - 1 do
    begin
      ScreenData[Index] := ScanLine[ColLoop].rgbtRed;
      ScreenData[Index + 1] := ScanLine[ColLoop].rgbtGreen;
      ScreenData[Index + 2] := ScanLine[ColLoop].rgbtBlue;
      Inc (Index,3);
    end;
  end;
end;


procedure TfrmMain.Timer3Timer ( Sender : TObject
                               );
begin
  { Clear the screen. }
  glClear (GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);
  { Set up for 2D rendering... }
  SetOrthographicProjection;
  { Draw the background Bitmap... }
  DrawBackground;
  { Set up for 3D rendering... }
  SetPerspectiveProjection;
  { Draw the numbers in the foreground... }
  DrawForeground;
  { Draw the whole thing... }
  SwapBuffers (wglGetCurrentDC);
end;


procedure TfrmMain.SetOrthographicProjection;
begin
  glViewport (0,0,ClientWidth,ClientHeight);
  glMatrixMode (GL_PROJECTION);
  glLoadIdentity;
  gluOrtho2D (0,ClientWidth,0,ClientHeight);
  glMatrixMode (GL_MODELVIEW);
  glLoadIdentity;
end;


procedure TfrmMain.SetPerspectiveProjection;
begin
  glViewport (0,0,ClientWidth,ClientHeight);
  glMatrixMode (GL_PROJECTION);
  glLoadIdentity;
  gluPerspective (45.0,ClientWidth / ClientHeight,0.1,100.0);
  glMatrixMode (GL_MODELVIEW);
  glLoadIdentity;
end;


procedure TfrmMain.DrawBackground;
begin
  { Start in the bottom-left corner. }
  glRasterPos2i (0,0);
  { Draw the saved Bitmap by pointing to the ScreenData array of Byte. }
  glDrawPixels (ClientWidth,ClientHeight,GL_RGB,GL_UNSIGNED_BYTE,@ScreenData);
end;


procedure TfrmMain.DrawForeground;
begin
  Inc (Changer,5);

  if ((Changer = 360) and (ToPrint = '3')) then
  begin
    glColor3f (1,1,0);
    ToPrint := '2';
    Changer := 0;
  end else
  if ((Changer = 360) and (ToPrint = '2')) then
  begin
    glColor3f (0,1,0);
    ToPrint := '1';
    Changer := 0;
  end else
  if ((Changer = 360) and (ToPrint = '1')) then
  begin
    glColor3f (1,0,0);
    ToPrint := '3';
    Changer := 0;
  end;

  glClear (GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);

  glTranslatef (0,-0.33,-1);
  glRotatef (Rotation,0,1.0,0.0);
  glRotatef (Sin (Changer / 600) * 10,0,1,0);
  glRotatef (Sin (Changer / 600) * 10,1,0,0);
  glPrint (ToPrint);
end;


procedure KillFont;
begin
  glDeleteLists (Base,256);
end;


procedure TfrmMain.bEndClick ( Sender : TObject
                             );
begin
  KillFont;
  Halt (0);
end;


end.
